/********************************************************
Workspace: IAR+SPL
Chip type: STM8S003K3 | STM8S103K3
Project  : Amplipulse HB lite
Date     : Mar,2019
Version  : 0
*********************************************************/
#include <stdbool.h>
#include <stdlib.h>
#include <math.h>
#include "stm8s.h"
#include "iostm003.c"

#include "bitset.c"
#include "defines.c"
#include "periph_init.c"
#include "display_work.c"
#include "kb.c"
#include "adc_work.c"
#include "pwm_work.c"

volatile u16 tim0, //display
             tim1, //sigtimer
             tim2; //adc
volatile u32 proc_timer; //  

volatile bool iwdg_t1,iwdg_t4;


u8 pwm_vol;
bool sw_gen_trg;
u8 *wm_pt;
u8 *wm_npt;

u8 adc_result;
bool adc_ready;


void devcon();

extern "C"
{
 INTERRUPT_HANDLER(TIM1_X3, 11)
 {
  static bool restart_trg=1;
  static u8 *wm_tmp_pt=NULL;
  static u8 *wm_tmp_npt=NULL;

  //     
  if(sw_gen_trg)
  {
   //       
   if(wm_tmp_pt != wm_pt)
   {
    //    
    if(wave_make(wm_tmp_pt, wm_tmp_npt, pwm_vol)==0 || wm_tmp_pt==NULL)
    {
     wm_tmp_pt  = wm_pt;
     wm_tmp_npt = wm_npt;
    }
   }
   else wave_make(wm_tmp_pt, wm_tmp_npt, pwm_vol, restart_trg);

   restart_trg = 0;
  }
  else
  {
   //         
   if(!restart_trg)
   {
    if(wave_make(wm_pt, wm_npt, pwm_vol)==0) restart_trg = 1;
   }
   else TIM1->DTR = set_pwm(pwm_vol);
  }


  iwdg_t1=1;
  TIM1->SR1 = 0;
 }

 INTERRUPT_HANDLER(TIM4_OVF, 23)
 {
  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;

  if(TIM1->BKR & TIM1_BKR_MOE)
  {
   if(proc_timer)proc_timer--;  //   
   else TIM1->BKR = 0;          //     
  }

  iwdg_t4   = 1;
  TIM4->SR1 = 0;
 }
}

void main()
{
 cfg.gpio();
 cfg.clk();
 cfg.t1();
 cfg.t4();
 cfg.adc();

#ifndef debug
 cfg.iwdg();
#endif

 ITC_SetSoftwarePriority(ITC_IRQ_TIM1_OVF, ITC_PRIORITYLEVEL_1);
 ITC_SetSoftwarePriority(ITC_IRQ_TIM4_OVF, ITC_PRIORITYLEVEL_0);

 detect_7seg_polarity(&ssg_pl);
 init_dpins(seg_wire, com_wire);

 asm("rim");

m:
 {
  devcon();

  //   ,   
  //   
  if(!tim2 && !adc_ready)
  {
   if(read_adc(&adc_result)==true)
   {
    tim2 = 100;
    adc_ready = 1;
   }
  }
  else ADC1->CR1 &= ~ADC1_CR1_ADON;
  //    

  //     
  if(iwdg_t4)
  {
   static bool trg;

   trg ^=1;
   if(trg) read_btn(&GPIO_btn);
   else    display_data();
  }
  //    

  //  
  if(!tim0)
  {
   tim0 = 400;

   if(proc_timer) ttd(dbuf, MAX_WORK_TIME - proc_timer);
   else           ttd(dbuf, 0);
   inwork_animation((dbuf+3), !!(TIM1->BKR & TIM1_BKR_MOE));
  }
  //  

  if(iwdg_t1 && iwdg_t4){iwdg_t1=0;iwdg_t4=0;IWDG_ReloadCounter();}
 }
goto m;
}


void devcon()
{
 static bool itr=0;
 static bool btn_trg;
 static bool bh_trg;
 static bool lb_trg;
 static enum dst{start=0,CMT_1,CMT_2,CMT_3,CMT_4,CMT_5,CMT_6,off} dev_st=start;
 static bool start_trg=0;

 //    
 if(btn == rls)
 {
  btn_trg = 0;
  bh_trg  = 0;

  //       
  if(lb_trg)
  {
   //      
   lb_trg = 0;

   if(dev_st < CMT_6)
   {
    u8 tmp = (u8)dev_st;
    tmp++;
    dev_st = (dst)tmp;
   }
   else dev_st = CMT_1;

   itr=0;
  }
 }
 else if(btn == prs && !btn_trg)
 {
  btn_trg = 1;
  //       
  if((TIM1->BKR & TIM1_BKR_MOE)==0)lb_trg = 1;

  //       
  TIM1->BKR = 0;
 }
 //        
 else if(btn == hold && !bh_trg)
 {
  bh_trg = 1;
  lb_trg = 0;

  //   
  if(dev_st != off)
  {
   //    
   if(adc_result < 0xff/10)
   {
    //       
    if(dev_st>start && dev_st<off)
    {
     start_trg = 1;
     proc_timer = MAX_WORK_TIME;
     TIM1->BKR = TIM1_BKR_MOE;
    }
   }
  }
  //     
  else dev_st = start;
 }

 //     
 if(start_trg && !proc_timer)
 {
  start_trg = 0;
  dev_st = off;
 }


 switch (dev_st)
 {
  case start:
  {
   if(!itr)
   {
    itr=1;

    TIM1->BKR = 0;
    adc_ready = 0;
   }

   dev_st = CMT_1;

   if(dev_st!=start){itr=0;}
  }
  break;

  // 1 - 5   
  //   
  case CMT_1:
  {
   if(!itr)
   {
    itr=1;
    dbuf[3]='P';dbuf[2]= 1;dbuf[1] = 0;dbuf[0] = 0;
    dpp[3] = 0; dpp[2] = 1; dpp[1] = 0; dpp[0] = 0;

    //    
    wm_pt = wave_mss_a;
    wm_npt = &mss_a_n;

    *wm_npt = calculate_wave(wm_pt, 150, 25);

    // 
    sw_gen_trg = true;
   }

   if(adc_ready)
   {
    adc_ready = 0;
    pwm_vol = adc_result;
   }


   if(dev_st!=CMT_1){itr=0;}
  }
  break;

  // 2 -   
  //  
  //         
  case CMT_2:
  {
   if(!itr)
   {
    itr=1;
    dbuf[3]='P';dbuf[2]= 2;dbuf[1] = 0;dbuf[0] = 0;
    dpp[3] = 0; dpp[2] = 1; dpp[1] = 0; dpp[0] = 0;

    //    
    wm_pt = wave_mss_a;
    wm_npt = &mss_a_n;

    *wm_npt = calculate_wave(wm_pt, 100, 50);
   }

   if(!tim1)
   {
    tim1 = 10;

    sw_gen_trg ^=1;

    if(!sw_gen_trg) pwm_vol = 0;
    else pwm_vol = adc_result;
   }
   else if(adc_ready)
   {
    adc_ready = 0;

    pwm_vol = adc_result;
   }


   if(dev_st!=CMT_2){itr=0;}
  }
  break;

  // 3 -     
  //  100,  50%,  3
  //  
  case CMT_3:
  {
   if(!itr)
   {
    itr=1;
    dbuf[3]='P';dbuf[2]= 3;dbuf[1] = 0;dbuf[0] = 0;
    dpp[3] = 0; dpp[2] = 1; dpp[1] = 0; dpp[0] = 0;

    //    
    wm_pt = wave_mss_a;
    wm_npt = &mss_a_n;

    *wm_npt = calculate_wave(wm_pt, 100, 50);
   }

   //     
   if(adc_ready)
   {
    pwm_vol = adc_result;

    adc_ready = 0;

    // /     
    if(!tim1)
    {
     tim1 = 3000;
     sw_gen_trg ^=1;
    }
   }



   if(dev_st!=CMT_3){itr=0;}
  }
  break;

  // 4 -    ,  150
  //    150    ,
  //     
  case CMT_4:
  {
   if(!itr)
   {
    itr=1;
    dbuf[3]='P';dbuf[2]= 4;dbuf[1] = 0;dbuf[0] = 0;
    dpp[3] = 0; dpp[2] = 1; dpp[1] = 0; dpp[0] = 0;

    //     
    wm_pt = wave_mss_b;
    wm_npt = &mss_b_n;

    *wm_npt = calculate_wave(wm_pt, 75, 50);

    //     
    wm_pt = wave_mss_a;
    wm_npt = &mss_a_n;

    *wm_npt = calculate_wave(wm_pt, 150, 50);

    // 
    sw_gen_trg = 1;
   }

   //     
   if(adc_ready)
   {
    pwm_vol = adc_result;

    adc_ready = 0;
   }

   //  
   static bool wave_sw_trg;
   {
    tim1 = 100;
    wave_sw_trg ^= 1;

    if(wave_sw_trg)
    {
     wm_pt  = wave_mss_a;
     wm_npt = &mss_a_n;
    }
    else
    {
     wm_pt  = wave_mss_b;
     wm_npt = &mss_b_n;
    }
   }



   if(dev_st!=CMT_4){itr=0;}
  }
  break;

  // 5 -  4   
  case CMT_5:
  {
   if(!itr)
   {
    itr=1;
    dbuf[3]='P';dbuf[2]= 5;dbuf[1] = 0;dbuf[0] = 0;
    dpp[3] = 0; dpp[2] = 1; dpp[1] = 0; dpp[0] = 0;

    //     
    wm_pt = wave_mss_b;
    wm_npt = &mss_b_n;

    *wm_npt = calculate_wave(wm_pt, 75, 50);

    //     
    wm_pt = wave_mss_a;
    wm_npt = &mss_a_n;

    *wm_npt = calculate_wave(wm_pt, 150, 50);
   }

   //     
   if(adc_ready)
   {
    pwm_vol = adc_result;

    adc_ready = 0;
   }

   // /
   static bool sp_trg;

   if(!tim1)
   {
    tim1 = 100;

    sp_trg ^=1;

    ///
    if(sp_trg)
    {
     //  
     static bool wave_sw_trg;

     wave_sw_trg ^= 1;

     if(wave_sw_trg)
     {
      wm_pt  = wave_mss_a;
      wm_npt = &mss_a_n;
     }
     else
     {
      wm_pt  = wave_mss_b;
      wm_npt = &mss_b_n;
     }

     sw_gen_trg = 1;
    }
    else
    {
     sw_gen_trg = 0;
     pwm_vol    = 0;
    }
   }

   if(dev_st!=CMT_5){itr=0;}
  }
  break;

  // 6 -   
  case CMT_6:
  {
   if(!itr)
   {
    itr=1;
    dbuf[3]='P';dbuf[2]= 6;dbuf[1] = 0;dbuf[0] = 0;
    dpp[3] = 0; dpp[2] = 1; dpp[1] = 0; dpp[0] = 0;

    // 
    sw_gen_trg = 0;
   }

   //    
   if(adc_ready)
   {
    pwm_vol = adc_result;

    adc_ready = 0;
   }


   if(dev_st!=CMT_6){itr=0;}
  }
  break;

  case off:
  {
   if(!itr)
   {
    itr=1;
    dbuf[3]='O';dbuf[2]='F';dbuf[1]='F';dbuf[0]=' ';
    dpp[3] = 0; dpp[2] = 0; dpp[1] = 0; dpp[0] = 0;

    TIM1->BKR = 0;
   }




   if(dev_st!=off){itr=0;}
  }
  break;

  default:break;
 }
}

