#define DT_NS      u8((u32)1E9/CPU_FRQ)   //125ns      8
#define PULSE_TIME u8((u32)1E6/CAR_FRQ/2) //100us  5

#define DT_S3  (u8)((u32)(32+bin(00011111))*16*DT_NS/1000) //126us
#define DT_S2  (u8)((u32)(32+bin(00011111))*8 *DT_NS/1000) //63us
#define DT_S1  (u8)((u32)(64+bin(00111111))*2 *DT_NS/1000) //32us
#define DT_S0  (u8)((u32)    bin(01111111)    *DT_NS/1000) //16us

#define DT_MIN 5 //     5us
#define DTR_MIN  DT_MIN*1000/DT_NS //    DTR

#define MSS_SIZE 0xff/2
//    
u8 wave_mss_a[MSS_SIZE];
u8 mss_a_n;
u8 wave_mss_b[MSS_SIZE];
u8 mss_b_n;

//     DTR    (0..0xff%)
//(     5)
u8 set_pwm(u8 val)
{
 static u8 tmp_val=0xff/10;
 static u8 dt_retval;

 if(tmp_val!=val)
 {
  u8 dt_tmp;

  tmp_val = val;

  //  us
  dt_tmp = PULSE_TIME - ((u16)tmp_val*PULSE_TIME>>8);

  if(dt_tmp>=PULSE_TIME) dt_retval = 0xff;
  else if(dt_tmp > DT_S2)
  {
   dt_retval = bin(111)<<5|(u8)((u32)dt_tmp*1000/(16*DT_NS)-32);
  }
  else if(dt_tmp > DT_S1)
  {
   dt_retval = bin(110)<<5|(u8)((u32)dt_tmp*1000/(8*DT_NS)-32);
  }
  else if(dt_tmp > DT_S0)
  {
   dt_retval = bin(100)<<5|(u8)((u32)dt_tmp*1000/(2*DT_NS)-64);
  }
  else
  {
   dt_retval = (u32)dt_tmp*1000/DT_NS;
  }
 }

 if(dt_retval < DTR_MIN) return DTR_MIN;

 return dt_retval;
}

//    ,   
//      
u8 calculate_wave(u8 wave_mss[], u8 m_frq, u8 m_depth)
{
 u8 n=0;
 u8 tmp_depth;
 u8 pwm_floor;

 //    
 if(m_frq < CAR_FRQ/2/MSS_SIZE) m_frq = CAR_FRQ/2/MSS_SIZE+1;

 //     
 n = CAR_FRQ/m_frq/2;

if(n > MSS_SIZE-1) n = MSS_SIZE-1;

 //       100% 
 if(m_depth == 0)
 {
  for(u8 i=0;i<n;i++) wave_mss[i]=0xff;
  return n;
 }

 //       
 tmp_depth = ((u16)m_depth*0xff)/100;

 //    
 pwm_floor = 0xff - tmp_depth;

 //      
 for(u8 i=0;i<n;i++)
 {
  wave_mss[i] = (u8)(tmp_depth*sin(((3.14/2)*i)/(n-1)))+pwm_floor;
 }

 return n;
}

//   DTR     
//     (0..0xff)
//     
u8 wave_make(const u8 wave_mss[], const u8 *mss_n, u8 vol=0xff, bool restart=false)
{
 static u8 n_cnt=0;
 static bool wave_dir;
// static bool cd_trg;

 if(restart == true)
 {
  n_cnt = 0;
  wave_dir = 0;
 }

 if(!n_cnt && !wave_dir)
 {
  wave_dir = 1;

 }
 else if(n_cnt >= *mss_n-1)
 {
  n_cnt = *mss_n-1;
  if(wave_dir)
  {
   wave_dir = 0;

  }
 }

 u8 tmp = (u16)wave_mss[n_cnt]*vol >>8;

 TIM1->DTR = set_pwm(tmp);

 if(wave_dir) n_cnt++;
 else         n_cnt--;

 return n_cnt;
}

//         
//    
void declivity_make(const u8 start_pwm_val,
                    const u8 end_pwm_val,
                    const u16 time_ms,
                    u8 *time_step,
                    s8 *pwm_step)
{
 s16 tmp;

 tmp = end_pwm_val - start_pwm_val;

 //      
 //     
 if(abs(tmp) > time_ms)
 {
  *time_step = 1;
  *pwm_step = tmp/time_ms;
 }
 else
 {
  *time_step = time_ms/ *pwm_step;

  if(tmp<0) *pwm_step = -1;
  else      *pwm_step = 1;
 }
}


//_________________________________________